# This script is made by William Au Cheuk Wang (2024) to process Rosetta alanine scan data. This is the mutant script where the means from the WT plot is subtracted from the data and shown in violin plots.

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np

# Read the data from the CSV file
data = pd.read_csv('mutate-output_all.csv')

# Extract the column names
column_names = data.columns.tolist()

# Extract the data values
data_values = data.values

# Subtract 7.18 from each element in the data_values array
data_values -= 7.20

# Calculate the means and standard deviations of each column after the subtraction
column_means = np.mean(data_values, axis=0)
column_stds = np.std(data_values, axis=0)

# Create a figure and axes with adjusted figure size
fig, ax = plt.subplots(figsize=(len(column_names)*7.5, 9))

# Calculate the positions for each violin plot
positions = np.arange(len(column_names))

# Create a violin plot for each column
violin_parts = []
for i in range(len(column_names)):
    violin = ax.violinplot(data_values[:, i], positions=[i], showmedians=True)

    # Remove caps (cmaxes and cmins)
    for body_part in ['cmaxes', 'cmins']:
        violin[body_part].set_visible(False)

    violin_parts.append(violin)

    # Print the mean above each violin plot with a smaller font size
    ax.text(i, column_means[i], f'{column_means[i]:.2f}', ha='center', va='bottom', fontweight='bold', fontsize=7.5)

    # Print the standard deviation in the terminal
    print(f"Standard Deviation for '{column_names[i]}': {column_stds[i]:.2f}")

    # Add standard deviation ticks to the violin plot
    std_tick_height = 0.05
    ax.hlines(y=column_means[i] - column_stds[i], xmin=i - 0.2, xmax=i + 0.2, color='black')
    ax.hlines(y=column_means[i] + column_stds[i], xmin=i - 0.2, xmax=i + 0.2, color='black')
    ax.vlines(x=i - 0.2, ymin=column_means[i] - column_stds[i], ymax=column_means[i] - column_stds[i] + std_tick_height, color='black')
    ax.vlines(x=i + 0.2, ymin=column_means[i] - column_stds[i], ymax=column_means[i] - column_stds[i] + std_tick_height, color='black')

# Set the x-axis labels
ax.set_xticks(positions)
ax.set_xticklabels(column_names, rotation=45)

# Set the y-axis label
ax.set_ylabel('\u0394REU')

# Draw a diagonal line at y=0
ax.axhline(y=0, color='black', linestyle='--')

# Set the same color for all violin plots
color = 'grey'

# Adjust the style and spacing of the violin plots
for pc in violin_parts:
    pc['bodies'][0].set_alpha(0.2)  # Set transparency
    pc['bodies'][0].set_edgecolor('black')  # Set edge color
    pc['bodies'][0].set_facecolor(color)  # Set fill color
    pc['bodies'][0].set_linewidth(1.5)  # Set line width
    pc['bodies'][0].set_linestyle('-')  # Set line style

# Save the plot to a file with increased resolution
plt.savefig('violin_plot.png')

# Show the plot
plt.show()

# Save means and standard deviations to a CSV file in the current directory
stats_df = pd.DataFrame({
    'Column': column_names,
    'Mean': column_means,
    'Standard Deviation': column_stds
})
stats_df.to_csv('./column_stats.csv', index=False)


