# This script is made by William Au Cheuk Wang (2024)

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np

# Read the data from the CSV file
data = pd.read_csv('mutate-output_all.csv')

# Extract the column names
column_names = data.columns.tolist()

# Extract the data values
data_values = data.values

# Create a figure and axes with adjusted figure size
fig, ax = plt.subplots(figsize=(len(column_names)*5, 6))

# Calculate the positions for each violin plot
positions = np.arange(len(column_names))

# Create a violin plot for each column
violin_parts = []
column_means = []  # List to store mean values of each column
for i in range(len(column_names)):
    violin = ax.violinplot(data_values[:, i], positions=[i], showmedians=True)

    # Remove caps (cmaxes and cmins)
    for body_part in ['cmaxes', 'cmins']:
        violin[body_part].set_visible(False)

    violin_parts.append(violin)

    # Calculate the mean for the current column
    column_mean = np.mean(data_values[:, i])
    column_means.append(column_mean)  # Store the mean value for each column

    # Print the mean above each violin plot with a smaller font size
    ax.text(i, column_mean, f'{column_mean:.2f}', ha='center', va='bottom', fontweight='bold', fontsize=6)

# Calculate the mean of all the means
mean_of_means = np.mean(column_means)

# Print the mean of all the means
print(f"Mean of all the means: {mean_of_means:.2f}")

# Set the x-axis labels
ax.set_xticks(positions)
ax.set_xticklabels(column_names, rotation=45)

# Set the y-axis label
ax.set_ylabel('\u0394REU')

# Draw a diagonal line at y=0
ax.axhline(y=0, color='black', linestyle='--')

# Set the same color for all violin plots
color = 'grey'

# Adjust the style of the violin plots
for pc in violin_parts:
    pc['bodies'][0].set_alpha(0.2)  # Set transparency
    pc['bodies'][0].set_edgecolor('black')  # Set edge color
    pc['bodies'][0].set_facecolor(color)  # Set fill color
    pc['bodies'][0].set_linewidth(1.5)  # Set line width
    pc['bodies'][0].set_linestyle('-')  # Set line style

# Save the plot to a file with increased resolution
plt.savefig('violin_plot.png')

# Show the plot
plt.show()
